import algoliasearch from "algoliasearch";
import fs from "fs";
import path from "path";
import { parseMarkdown } from "./compile-terms";

const termsFiles = fs.readdirSync(`${__dirname}/terms`);

const makeGlossaryTermId = (term: string) =>
  term.replace(/\s+/g, "-").replace(/[@/]/g, "").toLowerCase();

const stripMarkdown = (text: string) => {
  // Remove bold formatting (e.g., **bold**)
  text = text.replace(/\*\*(.*?)\*\*/g, "$1");
  // Remove italic formatting (e.g., *italic*)
  text = text.replace(/\*(.*?)\*/g, "$1");
  // Remove inline code formatting (e.g., `code`)
  text = text.replace(/`(.*?)`/g, "$1");
  // Remove links (e.g., [link text](url))
  text = text.replace(/\[(.*?)\]\((.*?)\)/g, "$1");
  return text;
};

const stripCodeExample = (definition: string) => {
  const hasCodeBlocks = /```[\s\S]*?```/g.test(definition);

  if (hasCodeBlocks) {
    // Find the index of the first occurrence of the code block
    const codeBlockIndex = definition.indexOf("```");

    if (codeBlockIndex !== -1) {
      // Capture the text before the code block
      const beforeCodeBlock = definition.substring(0, codeBlockIndex);

      // Remove everything after the code block
      definition = beforeCodeBlock.replace(/(\n|^)\s*$/, "");
    }

    // Remove the last sentence if it contains "example" or "shown below"
    const sentences = definition.split(".").map((sentence) => sentence.trim());
    const filteredSentences = sentences
      .filter((sentence) => sentence !== "")
      .filter(
        (sentence, i, arr) =>
          i !== arr.length - 1 ||
          (!sentence.includes("example") && !sentence.includes("shown below"))
      );

    definition = filteredSentences.join(". ").trim();
    if (!definition.endsWith(".")) {
      definition = definition += ".";
    }
  }

  return definition;
};

const terms = termsFiles
  .filter((file) => file.endsWith(".md"))
  .map((file) => {
    const term = parseMarkdown(`${__dirname}/terms/${file}`);
    const {
      labels,
      relatedTerms,
      usageInstructions,
      exampleUsage,
      businessContext,
      internalOnly,
      learnMore,
      learnMoreText,
    } = term.frontmatter;

    const termName = path.basename(file, path.extname(file));

    const url = `https://www.apollographql.com/docs/resources/glossary#${makeGlossaryTermId(
      decodeURIComponent(termName)
    )}`;

    const text = stripMarkdown(stripCodeExample(term.definition));
    const categories = ["glossary", ...(labels ?? [])];

    return {
      term: decodeURIComponent(termName),
      objectID: termName,
      definition: term.definition,
      text,
      labels,
      categories,
      relatedTerms,
      usageInstructions,
      exampleUsage,
      businessContext,
      internalOnly,
      learnMore,
      learnMoreText,
      url,
    };
  });

const settings: AlgoliaIndexSettings = {
  searchableAttributes: ["term", "unordered(labels)"],
  attributesForFaceting: ["internalOnly", "labels"],
  attributesToSnippet: ["text:50"],
  attributesToHighlight: ["term", "text"],
  customRanking: ["asc(term)"],
};

// Initialize Algolia client
const client = algoliasearch(
  process.env.ALGOLIA_APP_ID!,
  process.env.ALGOLIA_API_KEY!
);

const indexName =
  process.env.GITHUB_REF_NAME === "main"
    ? "apollopedia"
    : "apollopedia_staging";

// Initialize the index
const index = client.initIndex(indexName);

// Set settings
index.setSettings(settings);

// Add or update records in the index
index
  .replaceAllObjects(terms)
  .then(({ objectIDs }) => {
    console.log(
      `Indexed ${objectIDs.length} terms to ${indexName} index in Algolia`
    );
  })
  .catch((error) => {
    console.error("Error indexing to Algolia:", error);
  });

interface AlgoliaIndexSettings {
  searchableAttributes: string[];
  attributesForFaceting: string[];
  attributesToSnippet: string[];
  attributesToHighlight: string[];
  customRanking: string[];
}
